package com.ximai.mes.kanban.service;

import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ximai.common.utils.data.StringUtils;
import com.ximai.mes.constant.QcAbnormalTypeEnum;
import com.ximai.mes.constant.TaskWorkunitStatusEnum;
import com.ximai.mes.kanban.dto.abnormal.AbnormalMonthStatDto;
import com.ximai.mes.kanban.dto.equipment.EquipmentQuery;
import com.ximai.mes.kanban.dto.equipment.EquipmentStatDto;
import com.ximai.mes.kanban.dto.task.TaskStatDto;
import com.ximai.mes.md.domain.MdWorkunit;
import com.ximai.mes.md.service.IMdWorkunitService;
import com.ximai.mes.md.vo.MdWorkunitVo;
import com.ximai.mes.pro.domain.ProFeedback;
import com.ximai.mes.pro.domain.task.ProTaskWorkunit;
import com.ximai.mes.pro.dto.ProFeedbackDto;
import com.ximai.mes.pro.dto.task.ProTaskWorkunitDto;
import com.ximai.mes.pro.mapper.task.ProTaskWorkunitMapper;
import com.ximai.mes.pro.service.IProFeedbackService;
import com.ximai.mes.pro.service.task.IProTaskWorkunitService;
import com.ximai.mes.qc.domain.QcAbnormalReport;
import com.ximai.mes.qc.dto.QcAbnormalReportDto;
import com.ximai.mes.qc.dto.QcAbnormalReportStatusEnum;
import com.ximai.mes.qc.service.IQcAbnormalReportService;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.ResponseBody;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
public class KanbanEquipmentService {

    @Autowired
    private IProTaskWorkunitService taskWorkunitService;
    @Autowired
    private IProFeedbackService feedbackService;
    @Autowired
    private ProTaskWorkunitMapper taskWorkunitMapper;
    @Autowired
    private IMdWorkunitService workunitService;
    @Autowired
    IQcAbnormalReportService abnormalReportService;

    @ApiOperation("设备统计")
    @PostMapping("/currentMonthStat")
    public EquipmentStatDto currentMonthStat(EquipmentQuery equipmentQuery) {
        EquipmentStatDto rst = new EquipmentStatDto();
        //查询所有设备
        List<MdWorkunit> list = workunitService.selectListByQw(new QueryWrapper<MdWorkunit>().eq("t1.enable_flag","Y")
                .orderByAsc("t1.serial","t1.create_time"));
        Map<String, List<MdWorkunit>> equipMap = list.stream().map(s->{
            s.setLineName(org.apache.commons.lang.StringUtils.defaultString(s.getLineName(),"未指定"));
            return s;
        }).collect(Collectors.groupingBy(s->s.getLineName()));
        List<MdWorkunit> tempList = null;
        if(StringUtils.isNotEmpty(equipmentQuery.getLineName())){
            tempList = equipMap.get(equipmentQuery.getLineName());
        }else{
            tempList = list;
        }
        Map<String,List<EquipmentStatDto.EquipmentInfo>> equipInfoMap = this.getEquipmentInfo(tempList);
        rst.setEquipMap(MapUtil.sort(equipInfoMap));
        //查询当日设备报异常
        LocalDateTime currDate = LocalDateTime.now().with(LocalTime.MIN);
        QueryWrapper<QcAbnormalReport> query = new QueryWrapper<>();
        query.ge("t1.create_time", currDate);
        query.eq("t1.abnormal_type", QcAbnormalTypeEnum.DEVI.getType());
        List<QcAbnormalReportDto> abnormalList = abnormalReportService.selectQcAbnormalReportJoinTaskWorkunitDtoList(query);
        List<EquipmentStatDto.AbnormalInfo> abnormalInfoList = this.getAbnormalInfo(tempList, abnormalList);
        rst.setAbnormalInfoList(abnormalInfoList);
        rst.setEquipNum(tempList.size());//总设备
        rst.setQualifiedRatio(new BigDecimal("55"));
        rst.setEquipAvailability(new BigDecimal("65"));//利用率
        rst.setTotalOutput(new BigDecimal("2000"));//总产量
        rst.setAbnormalNum(abnormalList.size());//异常数
        rst.setUnProcessNum(5);//历史待处理数
        return rst;
    }

    private Map<String,List<EquipmentStatDto.EquipmentInfo>> getEquipmentInfo(List<MdWorkunit> list){
        List<EquipmentStatDto.EquipmentInfo> rst = new ArrayList<>();
        list.forEach(s->{
            EquipmentStatDto.EquipmentInfo info = new EquipmentStatDto.EquipmentInfo();
            info.setEquipmentNo(s.getWorkunitCode());
            info.setEquipmentName(s.getWorkunitName());
            info.setOutputRatio(new BigDecimal("15"));
            info.setOutput(new BigDecimal(new Random().nextInt(100)));
            info.setState(new Random().nextInt(3));
            info.setLineName(s.getLineName());
            rst.add(info);
        });
        return rst.stream().collect(Collectors.groupingBy(s->s.getLineName()));
    }

    private List<EquipmentStatDto.AbnormalInfo> getAbnormalInfo(List<MdWorkunit> list, List<QcAbnormalReportDto> abnormalList){
        List<EquipmentStatDto.AbnormalInfo> rst = new ArrayList<>();
        abnormalList.forEach(s->{
            //未关闭
            if(QcAbnormalReportStatusEnum.NOT.getStatus().equals( s.getAbnormalStatus())||
                    QcAbnormalReportStatusEnum.SUBMIT.getStatus().equals( s.getAbnormalStatus())){
                //在查询设备范围内
                list.stream().filter(e->e.getWorkunitId().equals(s.getWorkunitId())).findFirst().ifPresent(e->{
                    EquipmentStatDto.AbnormalInfo temp = new EquipmentStatDto.AbnormalInfo();
                    temp.setEquipmentNo(e.getWorkunitCode());
                    temp.setEquipmentName(e.getWorkunitName());
                    temp.setState(s.getAbnormalStatus());
                    temp.setCause(s.getAbnormalReason());
                    rst.add(temp);
                });

            }
        });
        return rst;
    }

}
